@@ -1,7 +1,7 @@
 <?php
 /**
  * Copyright 2007 Maintainable Software, LLC
- * Copyright 2008-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2008-2017 Horde LLC (http://www.horde.org/)
  *
  * @author     Mike Naberezny <mike@maintainable.com>
  * @author     Derek DeVries <derek@maintainable.com>
@@ -126,16 +126,14 @@ class Horde_Db_Adapter_Pdo_Pgsql extends Horde_Db_Adapter_Pdo_Base
         $temp = explode(' ', trim($sql), 4);
         $table = str_replace('"', '', $temp[2]);
 
-        // Try an insert with 'returning id' if available (PG >= 8.2)
-        if ($this->supportsInsertWithReturning()) {
-            if (!$pk) {
-                list($pk, $sequenceName) = $this->pkAndSequenceFor($table);
-            }
-            if ($pk) {
-                $id = $this->selectValue($sql . ' RETURNING ' . $this->quoteColumnName($pk), $arg1, $arg2);
-                $this->resetPkSequence($table, $pk, $sequenceName);
-                return $id;
-            }
+        // Try an insert with 'returning id'
+        if (!$pk) {
+            list($pk, $sequenceName) = $this->pkAndSequenceFor($table);
+        }
+        if ($pk) {
+            $id = $this->selectValue($sql . ' RETURNING ' . $this->quoteColumnName($pk), $arg1, $arg2);
+            $this->resetPkSequence($table, $pk, $sequenceName);
+            return $id;
         }
 
         // If neither pk nor sequence name is given, look them up.
@@ -193,6 +191,10 @@ class Horde_Db_Adapter_Pdo_Pgsql extends Horde_Db_Adapter_Pdo_Base
     {
         $this->_config['adapter'] = 'pgsql';
 
+        if (!empty($this->_config['socket']) && !empty($this->_config['host'])) {
+            throw new Horde_Db_Exception('Can only specify host or socket, not both');
+        }
+
         // PDO for PostgreSQL does not accept a socket argument
         // in the connection string; the location can be set via the
         // "host" argument instead.
@@ -261,6 +263,8 @@ class Horde_Db_Adapter_Pdo_Pgsql extends Horde_Db_Adapter_Pdo_Base
 
     /**
      * Returns the current ID of a table's sequence.
+     *
+     * @todo Remove unused $table parameter.
      */
     protected function _lastInsertId($table, $sequenceName)
     {
