@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2014-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2014-2017 Horde LLC (http://www.horde.org/)
  *
  * @author   Jan Schneider <jan@horde.org>
  * @license  http://www.horde.org/licenses/bsd
@@ -17,26 +17,8 @@
  * @category Horde
  * @package  Db
  */
-class Horde_Db_Value_Text implements Horde_Db_Value
+class Horde_Db_Value_Text extends Horde_Db_Value_Lob
 {
-    /**
-     * Text value to be quoted
-     *
-     * @var string
-     * @since Horde_Db 2.1.0
-     */
-    public $value;
-
-    /**
-     * Constructor
-     *
-     * @param string $textValue
-     */
-    public function __construct($textValue)
-    {
-        $this->value = $textValue;
-    }
-
     /**
      * @param Horde_Db_Adapter $db
      */
