@@ -1,7 +1,7 @@
 <?php
 /**
  * Copyright 2007 Maintainable Software, LLC
- * Copyright 2008-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2008-2017 Horde LLC (http://www.horde.org/)
  *
  * @author     Mike Naberezny <mike@maintainable.com>
  * @author     Derek DeVries <derek@maintainable.com>
@@ -204,6 +204,7 @@ abstract class Horde_Db_Adapter_TestBase extends Horde_Test_Case
     public function testInsertBlob()
     {
         $this->_createTable();
+        $columns = $this->_conn->columns('unit_tests');
 
         $result = $this->_conn->insertBlob(
             'unit_tests',
@@ -216,6 +217,12 @@ abstract class Horde_Db_Adapter_TestBase extends Horde_Test_Case
             7
         );
         $this->assertEquals(7, $result);
+        $this->assertEquals(
+            str_repeat("\0", 5000),
+            $columns['blob_value']->binaryToString($this->_conn->selectValue(
+                'SELECT blob_value FROM unit_tests WHERE id = 7'
+            ))
+        );
 
         $result = $this->_conn->insertBlob(
             'unit_tests',
@@ -228,8 +235,55 @@ abstract class Horde_Db_Adapter_TestBase extends Horde_Test_Case
             8
         );
         $this->assertEquals(8, $result);
+        $this->assertEquals(
+            str_repeat('X', 5000),
+            $columns['text_value']->binaryToString($this->_conn->selectValue(
+                'SELECT text_value FROM unit_tests WHERE id = 8'
+            ))
+        );
+
+        $stream = fopen('php://temp', 'r+');
+        fwrite($stream, str_repeat('X', 5000));
+        $result = $this->_conn->insertBlob(
+            'unit_tests',
+            array(
+                'id' => 9,
+                'integer_value' => 1001,
+                'text_value' => new Horde_Db_Value_Text($stream)
+            ),
+            null,
+            9
+        );
+        $this->assertEquals(9, $result);
+        $this->assertEquals(
+            str_repeat('X', 5000),
+            $columns['text_value']->binaryToString($this->_conn->selectValue(
+                'SELECT text_value FROM unit_tests WHERE id = 9'
+            ))
+        );
+
+        $stream = fopen('php://temp', 'r+');
+        fwrite($stream, str_repeat("\0", 10000));
+        $result = $this->_conn->insertBlob(
+            'unit_tests',
+            array(
+                'id' => 10,
+                'integer_value' => 1002,
+                'blob_value' => new Horde_Db_Value_Binary($stream)
+            ),
+            null,
+            10
+        );
+        $this->assertEquals(10, $result);
+        $this->assertEquals(
+            str_repeat("\0", 10000),
+            $columns['blob_value']->binaryToString($this->_conn->selectValue(
+                'SELECT blob_value FROM unit_tests WHERE id = 10'
+            ))
+        );
     }
 
+
     public function testUpdate()
     {
         $this->_createTable();
@@ -261,6 +315,28 @@ abstract class Horde_Db_Adapter_TestBase extends Horde_Test_Case
             'id = 1'
         );
         $this->assertEquals(1, $result);
+
+        $stream = fopen('php://temp', 'r+');
+        fwrite($stream, str_repeat('X', 5001));
+        $result = $this->_conn->updateBlob(
+            'unit_tests',
+            array(
+                'text_value' => new Horde_Db_Value_Text($stream)
+            ),
+            'id = 1'
+        );
+        $this->assertEquals(1, $result);
+
+        $stream = fopen('php://temp', 'r+');
+        fwrite($stream, str_repeat("\0", 5001));
+        $result = $this->_conn->updateBlob(
+            'unit_tests',
+            array(
+                'blob_value' => new Horde_Db_Value_Binary($stream)
+            ),
+            'id = 1'
+        );
+        $this->assertEquals(1, $result);
     }
 
     public function testDelete()
@@ -412,7 +488,7 @@ abstract class Horde_Db_Adapter_TestBase extends Horde_Test_Case
         $this->assertEquals(3, count($indexes));
 
         // sort by name so we can predict the order of indexes
-        usort($indexes, create_function('$a, $b', 'return strcmp($a->name, $b->name);'));
+        usort($indexes, function ($a, $b) { return strcmp($a->name, $b->name); });
 
         // multi-column index
         $col = array('integer_value', 'string_value');
